﻿using System.ComponentModel;
using System.Linq;
using FileSynchronizer.DataAccess;

namespace FileSynchronizer.Objects
{
    //---------------------------------------------------------------------------------------------------------------------------------------
    // Code based upon the State Pattern, as described by the GOF
    //
    // http://en.wikipedia.org/wiki/State_pattern and http://en.wikipedia.org/wiki/Talk:State_pattern#Disputed
    //
    // This is a Concrete State Object
    //
    //---------------------------------------------------------------------------------------------------------------------------------------
   
    class ProgramState_AutoSync : ProgramState
    {
        #region Private variables
        //---------------------------------------------------------------------------------------------------------------------------------------
        private RunWorkerCompletedEventHandler myExternalCompletionHandler;
        //---------------------------------------------------------------------------------------------------------------------------------------
        #endregion

        #region Constructors
        //---------------------------------------------------------------------------------------------------------------------------------------
        public ProgramState_AutoSync(ProgramContext TheProgramContext)
            : base(TheProgramContext)
        {
        }
        //---------------------------------------------------------------------------------------------------------------------------------------
        #endregion

        #region Public Methods
        //---------------------------------------------------------------------------------------------------------------------------------------
        // Called when we want to Synchronize all FolderPairs
        //---------------------------------------------------------------------------------------------------------------------------------------
        public override bool SynchronizeAllFolderPairs(IQueryable<FolderPair> TheFolderPairs, RunWorkerCompletedEventHandler ExternalCompletionHandler)
        {
            bool OperationResult = false;
            myExternalCompletionHandler = ExternalCompletionHandler;

            if (TheFolderPairs != null)
            {
                if (TheFolderPairs.ToList().Count() > 0)
                {
                    SyncFolderPairListHelper mySyncFolderPairListHelper = new SyncFolderPairListHelper(TheFolderPairs);

                    //TO DO: Add enterprise features like latency Checking

                    mySyncFolderPairListHelper.StartSync(SyncFolderPairListCompleted);
                    OperationResult = true;
                }
                else
                {
                    SyncFolderPairListCompleted(null, null);
                }
            }
            else
            {
                SyncFolderPairListCompleted(null, null);
            }

            return OperationResult;
        }

        //---------------------------------------------------------------------------------------------------------------------------------------
        // Callback Function which is called whenever the synchronization of ALL folderpairs is complete
        //---------------------------------------------------------------------------------------------------------------------------------------
        private void SyncFolderPairListCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ChangeState(new ProgramState_Idle(CurrentProgramContext));
            CurrentProgramContext.CurrentState = State.Idle;

            if (myExternalCompletionHandler != null)
            {
                myExternalCompletionHandler(sender, e);
            }
        }
        //---------------------------------------------------------------------------------------------------------------------------------------
        #endregion
    }
}
